#!/devtool/local/bin/perl
# imake           [vmake_options] targets
#
# @(#)imake     1997-11-27
#
# make targets
#
# OWN           root directory for complete release
# TOOL          root directory for tools
# WRK           root directory for production
# DBROOT        root directory for distributable files
# PCFLAGS       optional pascal compiler flags
# CCFLAGS       optional c compiler flags
# ASFLAGS       optional assembler flags
# LDFLAGS       optional linker flags
# KEEPFLG       optional keep intermediate files flag
# VMAKE_OPTION  optional make options
# VMAKE_PATH    search path for root directories
package imake;
use Env;
use Getopt::Long;

local $Target = "@ARGV";
%optctl = ();
GetOptions(\%optctl, "f", "q", "s", "t");

$USAGE = "usage : imake -{f|q|s} [-t] -- [vmake_options] target \n";

if ( $optctl{f} ) {
    # fast
    $ENV{VMAKE_VERSION} = "fast";
    $ENV{VMAKE_DEFAULT} = "fast";

}
elsif ( $optctl{q} ) {
    # quick
    $ENV{VMAKE_VERSION} = "quick";
    $ENV{VMAKE_DEFAULT} = "quick";
}
elsif ( $optctl{s} ) {
    # slow
    $ENV{VMAKE_VERSION} = "slow";
    $ENV{VMAKE_DEFAULT} = "quick";
}

die $USAGE unless ( defined($ENV{VMAKE_VERSION}) && @ARGV > 0 );
warn "\ncaution: usage of local profile!\n\n" if ! defined($GLOBAL_PROFILE);

do "$TOOLVARS";
die $@ if $@;

$ENV{BUILDPID} = "PID$$";
local $BuildNr = 0;
local ($IsSource, $FirstTarget, $Options, $LastTargetPart);
($FirstTarget) = $Target =~ /\s+([\w.\/]+(-[\w.\/]+)?)/;
if ( $FirstTarget )
{
    $Options = $`;
    if ( $Options =~ /c/ )
    {
        $Collection = 1;
    }
    $LastTargetPart = $';
    $IsSingleModule = ( $FirstTarget =~ m!^[vhgi]([a-z][a-z]+)\d\d+-?.*!i or
                        $FirstTarget =~ m!\.! )
             && ( $LastTargetPart =~ /^\s*$/ ) ? 1 : 0;
}
else
{
    $IsSingleModule = 1;
}
if ( $RELVER ne "TOOL" and $RELVER ne "R61" and !$IsSingleModule and
        $RELSTAT !~ /TEST/i  and !$Collection )
{
    local $cmd = "$TOOL/pgm/GetBuildNr \"$Options\" \"$FirstTarget$LastTargetPart\"";
    $BuildNr = "`$cmd`";
print ("\nCMD:",$BuildNr,"\n");
    if ( $BuildNr !~ m!BuildNr: (\d+)! )
    {
        warn "caution: Can't get a buildnumber. Don't use buildnumber.\n";
        $BuildNr = 0;
    }
    else
    {
        $BuildNr = $1;
        print "Build: ", $BuildNr, "\n";
    }
}
$ENV{BUILDNUMBER} = $BuildNr;
$ENV{BUILDSTRING} = FormatBuildNr($BuildNr);

# Sicherstellen, da ein altes Build-Modul entfernt wird
unlink <$WRK/fast/tmp/*_$ENV{BUILDPID}.*>;
unlink <$WRK/quick/tmp/*_$ENV{BUILDPID}.*>;
unlink <$WRK/slow/tmp/*_$ENV{BUILDPID}.*>;

do {
    $ENV{TOOLTRACE} = "on";
    local @ARGV = ( $TraceFile );
    do "$ENV{'TOOL'}/bin/closedoc$TOOLEXT";
    unlink $TraceFile;
} if defined($optctl{t});
ToolTrace(scalar localtime()."\timf @ARGV\n\n");
$rc = system("$TOOL/pgm/vmake @ARGV");
do {
    local @ARGV = ( "-r", $TraceFile );
    do "$TOOL/bin/opendoc$TOOLEXT";
} if defined($optctl{t});

# Sicherstellen, da ein altes Build-Modul entfernt wird
do {
    unlink <$WRK/fast/tmp/*_$ENV{BUILDPID}.*>;
    unlink <$WRK/quick/tmp/*_$ENV{BUILDPID}.*>;
    unlink <$WRK/slow/tmp/*_$ENV{BUILDPID}.*>;
} unless $Options =~ /K/;

if ( ( $RELVER ne "TOOL" and $RELVER ne "R61" ) && ( $BuildNr > 0 ) )
{
    local $rc = system("$TOOL/pgm/PutBuildPrt $ENV{BUILDNUMBER} $WRK/prot/Modules.m0");
    if ( $rc < 0 )
    {
        warn "caution: Can't put protocol.\n";
    }
}
caller() ? $rc : exit $rc ;


##################################################################################

sub FormatBuildNr {
    local $Buildnr = $_[0];
    $ENV{BUILDPRAEFIX} = 0 unless defined($ENV{BUILDPRAEFIX});
    $Buildnr = "0" x (9-length($Buildnr)) . "$Buildnr";
    $Buildnr = join "-", grep /\d+/, split( /(\d\d\d)/, $Buildnr);
    return sprintf("%.3d", $ENV{BUILDPRAEFIX}) . "-". $Buildnr;
}
